/*
 * GroupLens Common Utilities
 * Copyright © 2011 Regents of the University of Minnesota
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *  * Neither the name of the University of Minnesota nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software has been partly funded by NSF grant IIS 08-08692.
 */
package org.grouplens.common.cache;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Caches are volatile key-value storage objects that should be implemented in a
 * thread safe manner (this is relatively easy, since you can base most of it on
 * a {@link ConcurrentHashMap}. By being volatile, caches are free to discard
 * key-value pairs from their cache for any reason at any point in time.
 * Implementations may provide specific rules that determine when things can be
 * discarded. Keys use equality as determined by their {@link #equals(Object)}
 * method, similar to how most {@link Map maps} work.
 * 
 * @author Michael Ludwig
 * @param <K>
 * @param <V>
 */
public interface Cache<K, V> {
    /**
     * Return the value in the cache for the given key. The key must not be
     * null. The object returned may be null if there was no cache entry for
     * they key, or if the cache purged it for some reason. Future calls, even
     * in rapid succession, may have a different value.
     * 
     * @param key The key
     * @return The cached object, or null
     * @throws NullPointerException if key is null
     */
    public V get(K key);

    /**
     * Store the given value in the cache, that can be retrieved by using the
     * given key with {@link #get(K)}. The stored value may be purged or
     * overwritten at some future point depending on the underlying
     * implementation, since permanent storage is not guaranteed.
     * 
     * @param key The key
     * @param value The value
     * @throws NullPointerException if key or value are null
     */
    public void put(K key, V value);
}
