/*
 * GroupLens Common Utilities
 * Copyright © 2011 Regents of the University of Minnesota
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *  * Neither the name of the University of Minnesota nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software has been partly funded by NSF grant IIS 08-08692.
 */
package org.grouplens.common.cache;

/**
 * <p>
 * PurgingCache is an extension of Cache that provides a couple of useful
 * methods for when a Cache has a purging policy where key-value pairs can be
 * removed automatically from the cache. It adds the optional put method,
 * {@link #put(K, V, PurgeListener)}, to allow some notification of when a
 * stored item is purged.
 * </p>
 * <p>
 * Additionally, it is extremely likely that PurgingCaches need to manage a
 * Thread that purges the cache periodically so service methods to start and
 * stop the cache are provided. It is possible to restart a PurgingCache, and a
 * stopped cache can still be used as a conventional Cache (it may not have
 * automatic purging, though).
 * </p>
 * <p>
 * Implementations should ensure that {@link #get(K)} will not return values
 * that should have been purged by their internal rules, even if they use an
 * automatic mechanism to periodically clean the cache.
 * </p>
 * 
 * @author Michael Ludwig
 * @param <K>
 * @param <V>
 */
public interface PurgingCache<K, V> extends Cache<K, V> {
    /**
     * Store a value in the cache, that can be retrieved by the given key. If
     * the PurgeListener is non null, its onPurge() method will be called when
     * the value is removed from the cache. </p>
     * <p>
     * It is important that every call to this method use a new PurgeListener
     * because onPurge() does not take the value or key. This is to support the
     * use of caches that use soft or weak references, in which case the value
     * may not be available.
     * </p>
     * 
     * @param key
     * @param value
     * @param listener
     */
    public void put(K key, V value, PurgeListener listener);

    /**
     * Start or re-start the automatic purging of the cache. Purging will occur
     * in {@link #get(V)} regardless of if the automatic purging service is
     * started. If the service is already started, this does nothing.
     */
    public void start();

    /**
     * Stop the automatic purging of the cache. Purging will occur in
     * {@link #get(V)} even after the purging service has been stopped. If the
     * service is already stopped, this does nothing. This should not clear the
     * cache or disable the use of get() and put().
     */
    public void shutdown();
}
